<?php

declare(strict_types=1);

namespace Drupal\refreshless_turbo\Hooks;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\hux\Attribute\Alter;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Library hook implementations.
 */
class Library implements ContainerInjectionInterface {

  /**
   * Hook constructor; saves dependencies.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The Drupal module handler.
   */
  public function __construct(
    protected readonly ModuleHandlerInterface $moduleHandler,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
    );
  }

  #[Alter('library_info')]
  /**
   * Alter library definitions to add our Ajax compatibility to core's Ajax.
   *
   * This adds our library as a dependency to 'core/drupal.ajax' so that our
   * library is always attached when core's is.
   *
   * @see \hook_library_info_alter()
   */
  public function alterCoreAjax(
    array &$libraries, string $extension,
  ): void {

    if ($extension !== 'core') {
      return;
    }

    $libraries['drupal.ajax']['dependencies'][] = 'refreshless_turbo/ajax';

  }

  #[Alter('library_info')]
  /**
   * Replaces core/misc/batch.js with our own.
   *
   * @see \hook_library_info_alter()
   */
  public function alterCoreBatch(
    array &$libraries, string $extension,
  ): void {

    if (
      $extension !== 'core' ||
      // Don't do anything if core's batch.js has been altered or replaced.
      !isset($libraries['drupal.batch']['js']['misc/batch.js'])
    ) {
      return;
    }

    // Copy the existing library definition from core's to ours.
    $libraries['drupal.batch']['js'][
      '/' . $this->moduleHandler->getModule('refreshless_turbo')->getPath() .
      '/js/overrides/batch.js'
    ] = $libraries['drupal.batch']['js']['misc/batch.js'];

    // Remove the original JavaScript entry.
    unset($libraries['drupal.batch']['js']['misc/batch.js']);

  }

  #[Alter('library_info')]
  /**
   * Replace Drupal core's 'misc/drupalSettingsLoader.js' with our own.
   *
   * @see \hook_library_info_alter()
   *
   * @see https://www.drupal.org/project/refreshless/issues/3414538
   *   JavaScript aggregation issue.
   *
   * @see https://www.drupal.org/project/refreshless/issues/3399243
   *   Tracking issue for core/contrib fixes.
   */
  public function alterCoreDrupalSettingsLoader(
    array &$libraries, string $extension,
  ): void {

    if (
      $extension !== 'core' ||
      // Don't do anything if the core JavaScript has been altered by another
      // extension.
      !isset($libraries['drupalSettings']['js']['misc/drupalSettingsLoader.js'])
    ) {
      return;
    }

    // Copy the existing library definition from core's to ours.
    $libraries['drupalSettings']['js'][
      '/' . $this->moduleHandler->getModule('refreshless_turbo')->getPath() .
      '/js/overrides/drupal_settings_loader.js'
    ] = $libraries['drupalSettings']['js']['misc/drupalSettingsLoader.js'];

    // Remove the original JavaScript entry.
    unset($libraries['drupalSettings']['js']['misc/drupalSettingsLoader.js']);

    // We use once() on the <html> element to ensure our settings loader only
    // runs once even when evaluated multiple times via different aggregates on
    // the same page.
    $libraries['drupalSettings']['dependencies'][] = 'core/once';

  }

  #[Alter('library_info')]
  /**
   * Alter library definitions to add our Gin integration to that theme.
   *
   * This adds our library as a dependency to 'gin/gin_accent' so that our
   * library is always attached when Gin's is.
   *
   * @see \hook_library_info_alter()
   */
  public function alterGin(
    array &$libraries, string $extension,
  ): void {

    if ($extension !== 'gin') {
      return;
    }

    $libraries['gin_accent']['dependencies'][] = 'refreshless_turbo/gin_accent';

  }

}
