<?php

declare(strict_types=1);

namespace Drupal\refreshless_turbo\Hooks;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Theme\ThemeManagerInterface;
use Drupal\hux\Attribute\Hook;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Page attachments hook implementations.
 */
class PageAttachments implements ContainerInjectionInterface {

  /**
   * Hook constructor; saves dependencies.
   *
   * @param \Drupal\Core\Theme\ThemeManagerInterface $themeManager
   *   The Drupal theme manager.
   */
  public function __construct(
    protected readonly ThemeManagerInterface $themeManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('theme.manager'),
    );
  }

  #[Hook('page_attachments')]
  /**
   * Attaches RefreshLess to the page.
   *
   * @see \hook_page_attachments()
   */
  public function attach(array &$attachments): void {

    $attachments['#attached']['library'][] = 'refreshless_turbo/refreshless';

    // This outputs the machine name of the current theme along with the
    // data-turbo-track="reload" attribute which causes Turbo to automagically
    // do a full page load rather than a fetch when this element changes on
    // navigating to a new page.
    //
    // @see https://turbo.hotwired.dev/reference/attributes
    $attachments['#attached']['html_head'][] = [[
      '#type'       => 'html_tag',
      '#tag'        => 'meta',
      '#attributes' => [
        'name'              => 'refreshless-turbo-theme',
        'content'           => $this->themeManager->getActiveTheme()->getName(),
        'data-turbo-track'  => 'reload',
      ],
    ], 'refreshless_turbo_theme'];

    // This currently opts out of caching until we can reliably detach
    // behaviours before the <body> is cached by Turbo.
    //
    // @see https://turbo.hotwired.dev/handbook/building#opting-out-of-caching
    //
    // @see https://www.drupal.org/project/refreshless/issues/3411449
    $attachments['#attached']['html_head'][] = [[
      '#type'       => 'html_tag',
      '#tag'        => 'meta',
      '#attributes' => [
        'name'        => 'turbo-cache-control',
        'content'     => 'no-cache',
      ],
    ], 'refreshless_turbo_cache'];

  }

}
