<?php

declare(strict_types=1);

namespace Drupal\refreshless_turbo\Service;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Asset\AssetResolverInterface;
use Drupal\Core\Asset\AttachedAssetsInterface;
use Drupal\Core\Language\LanguageInterface;

/**
 * Decorated asset resolver service.
 *
 * @see \Drupal\Core\Asset\AssetResolver
 *   Drupal core implementation.
 *
 * @see \Drupal\Core\Asset\AssetResolverInterface
 *
 * @see https://www.drupal.org/project/refreshless/issues/3397370
 *   Explains the rationale for including drupalSettings in the <body>.
 */
class AssetResolver implements AssetResolverInterface {

  /**
   * Service constructor; saves dependencies.
   *
   * @param \Drupal\Core\Asset\AssetResolverInterface $decorated
   *   The asset resolver service we decorate.
   */
  public function __construct(
    protected readonly AssetResolverInterface $decorated,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getCssAssets(
    AttachedAssetsInterface $assets,
    $optimize,
    LanguageInterface $language = null,
  ) {
    return $this->decorated->getCssAssets($assets, $optimize, $language);
  }

  /**
   * {@inheritdoc}
   *
   * This copies drupalSettings from the header to the footer so that we can
   * easily determine which are the correct settings for a page as Turbo
   * merges in the <head> contents without a straightforward way to listen or
   * intervene.
   */
  public function getJsAssets(
    AttachedAssetsInterface $assets,
    $optimize,
    LanguageInterface $language = null,
  ) {

    [
      $jsAssetsHeader,
      $jsAssetsFooter,
    ] = $this->decorated->getJsAssets($assets, $optimize, $language);

    if (isset($jsAssetsHeader['drupalSettings'])) {

      if (isset($jsAssetsFooter['drupalSettings'])) {

        // Merge on top of any existing footer settings using the same method
        // core's AssetResolver::getJsAssets() does, i.e. ensuring we pass true
        // as the second parameter to preserve integer keys.
        $jsAssetsFooter['drupalSettings'] = NestedArray::mergeDeepArray([
          $jsAssetsFooter['drupalSettings'],
          $jsAssetsHeader['drupalSettings'],
        ], true);

      } else {

        $jsAssetsFooter['drupalSettings'] = $jsAssetsHeader['drupalSettings'];

      }

    }

    return [$jsAssetsHeader, $jsAssetsFooter];

  }

}
