/**
 * @file
 * Parse inline JSON and initialize the drupalSettings global object.
 *
 * This is very similar to the Drupal core library, but with the following
 * changes:
 *
 * - Instead of wrapping the code in an IIFE (Immediately Invoked Function
 *   Expression), we wrap it in a once() call applied to the <html> element;
 *   this is to prevent re-evaluation if/when performing Turbo visits to pages
 *   where a different set of aggregates are generated that also contain this
 *   file. Core doesn't have to worry about this because the JavaScript
 *   environment gets destroyed when leaving a page, and so the old aggregates'
 *   contents are no longer in the environment, but with a persistent process we
 *   don't have that to clean up after leaving a page.
 *
 * - More documentation and more readable formatting/wrapping.
 *
 * @see https://www.drupal.org/project/refreshless/issues/3414538
 *   JavaScript aggregation issue.
 *
 * @see https://www.drupal.org/project/refreshless/issues/3399243
 *   Tracking issue for core/contrib fixes.
 */

once('refreshless-turbo-drupal-settings-loader', 'html').forEach(function(
  element,
) {

  /**
   * The drupalSettings <script> element selector.
   *
   * Note that this doesn't include the leading '>' combinator for XSS
   * hardening.
   *
   * @type {String}
   */
  const scriptSelector = 'script[type="application/json"][data-drupal-selector="drupal-settings-json"]';

  /**
   * The drupalSettings <script> element.
   *
   * Note the use of the '>' combinator to prevent XSS exploits when CSP is on.
   *
   * @type {HTMLElement}
   */
  const settingsElement = document.querySelector(
    `head > ${scriptSelector}, body > ${scriptSelector}`,
  );

  /**
   * Variable generated by Drupal with all the configuration created from PHP.
   *
   * @global
   *
   * @type {object}
   */
  window.drupalSettings = {};

  if (settingsElement !== null) {
    window.drupalSettings = JSON.parse(settingsElement.textContent);
  }

});
